/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.0 (build: 2013-06-26 16:27:34 UTC)
 *  on 2013-07-15 at 20:38:27 UTC 
 */

package com.google.api.services.prediction;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Prediction (v1.6).
 *
 * <p>
 * Lets you access a cloud hosted machine learning service that makes it easy to build smart apps
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/prediction/docs/developer-guide" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link PredictionRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Prediction extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Prediction API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "prediction/v1.6/projects/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Prediction(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Prediction(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Hostedmodels collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Prediction prediction = new Prediction(...);}
   *   {@code Prediction.Hostedmodels.List request = prediction.hostedmodels().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Hostedmodels hostedmodels() {
    return new Hostedmodels();
  }

  /**
   * The "hostedmodels" collection of methods.
   */
  public class Hostedmodels {

    /**
     * Submit input and request an output against a hosted model.
     *
     * Create a request for the method "hostedmodels.predict".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Predict#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param hostedModelName The name of a hosted model.
     * @param content the {@link com.google.api.services.prediction.model.Input}
     * @return the request
     */
    public Predict predict(String project, String hostedModelName, com.google.api.services.prediction.model.Input content) throws java.io.IOException {
      Predict result = new Predict(project, hostedModelName, content);
      initialize(result);
      return result;
    }

    public class Predict extends PredictionRequest<com.google.api.services.prediction.model.Output> {

      private static final String REST_PATH = "{project}/hostedmodels/{hostedModelName}/predict";

      /**
       * Submit input and request an output against a hosted model.
       *
       * Create a request for the method "hostedmodels.predict".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Predict#execute()} method to invoke the remote operation.
       * <p> {@link Predict#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param hostedModelName The name of a hosted model.
       * @param content the {@link com.google.api.services.prediction.model.Input}
       * @since 1.13
       */
      protected Predict(String project, String hostedModelName, com.google.api.services.prediction.model.Input content) {
        super(Prediction.this, "POST", REST_PATH, content, com.google.api.services.prediction.model.Output.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.hostedModelName = Preconditions.checkNotNull(hostedModelName, "Required parameter hostedModelName must be specified.");
      }

      @Override
      public Predict setAlt(String alt) {
        return (Predict) super.setAlt(alt);
      }

      @Override
      public Predict setFields(String fields) {
        return (Predict) super.setFields(fields);
      }

      @Override
      public Predict setKey(String key) {
        return (Predict) super.setKey(key);
      }

      @Override
      public Predict setOauthToken(String oauthToken) {
        return (Predict) super.setOauthToken(oauthToken);
      }

      @Override
      public Predict setPrettyPrint(Boolean prettyPrint) {
        return (Predict) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Predict setQuotaUser(String quotaUser) {
        return (Predict) super.setQuotaUser(quotaUser);
      }

      @Override
      public Predict setUserIp(String userIp) {
        return (Predict) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Predict setProject(String project) {
        this.project = project;
        return this;
      }

      /** The name of a hosted model. */
      @com.google.api.client.util.Key
      private String hostedModelName;

      /** The name of a hosted model.
       */
      public String getHostedModelName() {
        return hostedModelName;
      }

      /** The name of a hosted model. */
      public Predict setHostedModelName(String hostedModelName) {
        this.hostedModelName = hostedModelName;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Trainedmodels collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Prediction prediction = new Prediction(...);}
   *   {@code Prediction.Trainedmodels.List request = prediction.trainedmodels().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Trainedmodels trainedmodels() {
    return new Trainedmodels();
  }

  /**
   * The "trainedmodels" collection of methods.
   */
  public class Trainedmodels {

    /**
     * Get analysis of the model and the data the model was trained on.
     *
     * Create a request for the method "trainedmodels.analyze".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Analyze#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param id The unique name for the predictive model.
     * @return the request
     */
    public Analyze analyze(String project, String id) throws java.io.IOException {
      Analyze result = new Analyze(project, id);
      initialize(result);
      return result;
    }

    public class Analyze extends PredictionRequest<com.google.api.services.prediction.model.Analyze> {

      private static final String REST_PATH = "{project}/trainedmodels/{id}/analyze";

      /**
       * Get analysis of the model and the data the model was trained on.
       *
       * Create a request for the method "trainedmodels.analyze".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Analyze#execute()} method to invoke the remote operation.
       * <p> {@link Analyze#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param id The unique name for the predictive model.
       * @since 1.13
       */
      protected Analyze(String project, String id) {
        super(Prediction.this, "GET", REST_PATH, null, com.google.api.services.prediction.model.Analyze.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Analyze setAlt(String alt) {
        return (Analyze) super.setAlt(alt);
      }

      @Override
      public Analyze setFields(String fields) {
        return (Analyze) super.setFields(fields);
      }

      @Override
      public Analyze setKey(String key) {
        return (Analyze) super.setKey(key);
      }

      @Override
      public Analyze setOauthToken(String oauthToken) {
        return (Analyze) super.setOauthToken(oauthToken);
      }

      @Override
      public Analyze setPrettyPrint(Boolean prettyPrint) {
        return (Analyze) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Analyze setQuotaUser(String quotaUser) {
        return (Analyze) super.setQuotaUser(quotaUser);
      }

      @Override
      public Analyze setUserIp(String userIp) {
        return (Analyze) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Analyze setProject(String project) {
        this.project = project;
        return this;
      }

      /** The unique name for the predictive model. */
      @com.google.api.client.util.Key
      private String id;

      /** The unique name for the predictive model.
       */
      public String getId() {
        return id;
      }

      /** The unique name for the predictive model. */
      public Analyze setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Delete a trained model.
     *
     * Create a request for the method "trainedmodels.delete".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param id The unique name for the predictive model.
     * @return the request
     */
    public Delete delete(String project, String id) throws java.io.IOException {
      Delete result = new Delete(project, id);
      initialize(result);
      return result;
    }

    public class Delete extends PredictionRequest<Void> {

      private static final String REST_PATH = "{project}/trainedmodels/{id}";

      /**
       * Delete a trained model.
       *
       * Create a request for the method "trainedmodels.delete".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       * <p> {@link Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param id The unique name for the predictive model.
       * @since 1.13
       */
      protected Delete(String project, String id) {
        super(Prediction.this, "DELETE", REST_PATH, null, Void.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Delete setProject(String project) {
        this.project = project;
        return this;
      }

      /** The unique name for the predictive model. */
      @com.google.api.client.util.Key
      private String id;

      /** The unique name for the predictive model.
       */
      public String getId() {
        return id;
      }

      /** The unique name for the predictive model. */
      public Delete setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Check training status of your model.
     *
     * Create a request for the method "trainedmodels.get".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param id The unique name for the predictive model.
     * @return the request
     */
    public Get get(String project, String id) throws java.io.IOException {
      Get result = new Get(project, id);
      initialize(result);
      return result;
    }

    public class Get extends PredictionRequest<com.google.api.services.prediction.model.Insert2> {

      private static final String REST_PATH = "{project}/trainedmodels/{id}";

      /**
       * Check training status of your model.
       *
       * Create a request for the method "trainedmodels.get".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param id The unique name for the predictive model.
       * @since 1.13
       */
      protected Get(String project, String id) {
        super(Prediction.this, "GET", REST_PATH, null, com.google.api.services.prediction.model.Insert2.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Get setProject(String project) {
        this.project = project;
        return this;
      }

      /** The unique name for the predictive model. */
      @com.google.api.client.util.Key
      private String id;

      /** The unique name for the predictive model.
       */
      public String getId() {
        return id;
      }

      /** The unique name for the predictive model. */
      public Get setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Train a Prediction API model.
     *
     * Create a request for the method "trainedmodels.insert".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param content the {@link com.google.api.services.prediction.model.Insert}
     * @return the request
     */
    public Insert insert(String project, com.google.api.services.prediction.model.Insert content) throws java.io.IOException {
      Insert result = new Insert(project, content);
      initialize(result);
      return result;
    }

    public class Insert extends PredictionRequest<com.google.api.services.prediction.model.Insert2> {

      private static final String REST_PATH = "{project}/trainedmodels";

      /**
       * Train a Prediction API model.
       *
       * Create a request for the method "trainedmodels.insert".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       * <p> {@link Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param content the {@link com.google.api.services.prediction.model.Insert}
       * @since 1.13
       */
      protected Insert(String project, com.google.api.services.prediction.model.Insert content) {
        super(Prediction.this, "POST", REST_PATH, content, com.google.api.services.prediction.model.Insert2.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Insert setProject(String project) {
        this.project = project;
        return this;
      }

    }
    /**
     * List available models.
     *
     * Create a request for the method "trainedmodels.list".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @return the request
     */
    public List list(String project) throws java.io.IOException {
      List result = new List(project);
      initialize(result);
      return result;
    }

    public class List extends PredictionRequest<com.google.api.services.prediction.model.List> {

      private static final String REST_PATH = "{project}/trainedmodels/list";

      /**
       * List available models.
       *
       * Create a request for the method "trainedmodels.list".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @since 1.13
       */
      protected List(String project) {
        super(Prediction.this, "GET", REST_PATH, null, com.google.api.services.prediction.model.List.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public List setProject(String project) {
        this.project = project;
        return this;
      }

      /** Pagination token. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Pagination token.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Pagination token. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Maximum number of results to return. */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of results to return.

     [minimum: 0]
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }
    /**
     * Submit model id and request a prediction.
     *
     * Create a request for the method "trainedmodels.predict".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Predict#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param id The unique name for the predictive model.
     * @param content the {@link com.google.api.services.prediction.model.Input}
     * @return the request
     */
    public Predict predict(String project, String id, com.google.api.services.prediction.model.Input content) throws java.io.IOException {
      Predict result = new Predict(project, id, content);
      initialize(result);
      return result;
    }

    public class Predict extends PredictionRequest<com.google.api.services.prediction.model.Output> {

      private static final String REST_PATH = "{project}/trainedmodels/{id}/predict";

      /**
       * Submit model id and request a prediction.
       *
       * Create a request for the method "trainedmodels.predict".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Predict#execute()} method to invoke the remote operation.
       * <p> {@link Predict#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param id The unique name for the predictive model.
       * @param content the {@link com.google.api.services.prediction.model.Input}
       * @since 1.13
       */
      protected Predict(String project, String id, com.google.api.services.prediction.model.Input content) {
        super(Prediction.this, "POST", REST_PATH, content, com.google.api.services.prediction.model.Output.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Predict setAlt(String alt) {
        return (Predict) super.setAlt(alt);
      }

      @Override
      public Predict setFields(String fields) {
        return (Predict) super.setFields(fields);
      }

      @Override
      public Predict setKey(String key) {
        return (Predict) super.setKey(key);
      }

      @Override
      public Predict setOauthToken(String oauthToken) {
        return (Predict) super.setOauthToken(oauthToken);
      }

      @Override
      public Predict setPrettyPrint(Boolean prettyPrint) {
        return (Predict) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Predict setQuotaUser(String quotaUser) {
        return (Predict) super.setQuotaUser(quotaUser);
      }

      @Override
      public Predict setUserIp(String userIp) {
        return (Predict) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Predict setProject(String project) {
        this.project = project;
        return this;
      }

      /** The unique name for the predictive model. */
      @com.google.api.client.util.Key
      private String id;

      /** The unique name for the predictive model.
       */
      public String getId() {
        return id;
      }

      /** The unique name for the predictive model. */
      public Predict setId(String id) {
        this.id = id;
        return this;
      }

    }
    /**
     * Add new data to a trained model.
     *
     * Create a request for the method "trainedmodels.update".
     *
     * This request holds the parameters needed by the the prediction server.  After setting any
     * optional parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param project The project associated with the model.
     * @param id The unique name for the predictive model.
     * @param content the {@link com.google.api.services.prediction.model.Update}
     * @return the request
     */
    public Update update(String project, String id, com.google.api.services.prediction.model.Update content) throws java.io.IOException {
      Update result = new Update(project, id, content);
      initialize(result);
      return result;
    }

    public class Update extends PredictionRequest<com.google.api.services.prediction.model.Insert2> {

      private static final String REST_PATH = "{project}/trainedmodels/{id}";

      /**
       * Add new data to a trained model.
       *
       * Create a request for the method "trainedmodels.update".
       *
       * This request holds the parameters needed by the the prediction server.  After setting any
       * optional parameters, call the {@link Update#execute()} method to invoke the remote operation.
       * <p> {@link Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project The project associated with the model.
       * @param id The unique name for the predictive model.
       * @param content the {@link com.google.api.services.prediction.model.Update}
       * @since 1.13
       */
      protected Update(String project, String id, com.google.api.services.prediction.model.Update content) {
        super(Prediction.this, "PUT", REST_PATH, content, com.google.api.services.prediction.model.Insert2.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.id = Preconditions.checkNotNull(id, "Required parameter id must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** The project associated with the model. */
      @com.google.api.client.util.Key
      private String project;

      /** The project associated with the model.
       */
      public String getProject() {
        return project;
      }

      /** The project associated with the model. */
      public Update setProject(String project) {
        this.project = project;
        return this;
      }

      /** The unique name for the predictive model. */
      @com.google.api.client.util.Key
      private String id;

      /** The unique name for the predictive model.
       */
      public String getId() {
        return id;
      }

      /** The unique name for the predictive model. */
      public Update setId(String id) {
        this.id = id;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Prediction}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Prediction}. */
    @Override
    public Prediction build() {
      return new Prediction(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link PredictionRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setPredictionRequestInitializer(
        PredictionRequestInitializer predictionRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(predictionRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
